"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
/**
 * Converts a class constructor with parameters into a constructor with optional parameters.
 * Where possible parameter values are provided when no value is passed to the new constructor.
 * This logic is determied by the parameterProvider function.
 *
 * @param type Type to be constructed
 * @param parameterProvider function to lookup the constructor parameter value
 */
function injectConstructor(type, parameterProvider) {
    return createClassOptionalConstructorParams(type, parameterProvider);
}
exports.injectConstructor = injectConstructor;
function createClassOptionalConstructorParams(type, parameterProvider) {
    return wrapClassConstructor(type, parameterProvider);
}
function wrapClassConstructor(type, parameterProvider) {
    const params = Reflect.getMetadata("design:paramtypes", type);
    if (params == null || params.length == null || params.length === 0) {
        return type;
    }
    return class extends type {
        constructor(...args) {
            function resolveParameter(paramReflect, index) {
                return parameterProvider(args[index], paramReflect);
            }
            const constructorParams = params.map(resolveParameter);
            super(...constructorParams);
        }
    };
}
